import java.util.*;
public class PFSearch {
    Node[] nodes;
    public PFSearch(Node[] nodes) {
	this.nodes = nodes;
    }
    public void solve(Node src,Node dst) {
	// 未確定のノードを優先度キューで管理する
	PriorityQueue<Node> q = new PriorityQueue<Node>();
	// まず、全てのノードを未確定のノードとする。
	// ただし、距離無限大のノードはキューに入れない。
	for (int i=0; i<nodes.length; i++) {
	    nodes[i].value = Double.MAX_VALUE;	// 距離の初期値は無限大
	}
	// 始点ノードの距離を0と設定して、未確定ノード群に入れる。
	src.value = 0;	// 距離を0に書き換えて
	q.offer(src);	// 未確定ノード群に戻す
	while (q.size() > 0) {	// 未確定ノード群にノードがある限り繰り返す
	    Node x = q.poll();	// 距離最小ノードを未確定ノード群から取り出す
	    if (x.flag) continue; // 既に確定していたら
	    x.flag = true;	// xまでの最小距離が決定した
	    if (x.value == Double.MAX_VALUE) return; // 非連結ノード
	    if (x == dst) return; // 目的地までの距離が確定したら終了する
	    // 取り出したノードに隣接するノードの距離を書き換える
	    for (Edge e: x.edges) { // x.edgesの各要素が順にeに代入される
		Node y = e.dst; // xに隣接しているノードをyとする
		if (y.flag) continue; // 最小距離が確定しているyはスキップ
		double newValue = x.value +e.value; // x経由でのyまでの距離
		if (newValue < y.value) { // 新しい経路の方が近ければ
		    if (q.remove(y)) { // 可能ならば未確定ノード群から取り除き
			y.value = newValue; // yの距離を変更してから
			y.via = x; // x経由と記録して
			q.offer(y);	// yを未確定ノード群に戻す
		    }
		}
	    }
	}
    }
    public Node[] getPath(int id) {
	ArrayList<Node> v = new ArrayList<Node>();
	for (Node node = nodes[id]; node != null; node=node.via) v.add(node);
	for (int i=0; i<v.size()/2; i++) { // 逆順なので順番を逆に戻す
	    int j=v.size()-1-i;
	    Node tmp=v.get(i);
	    v.set(i,v.get(j));
	    v.set(j,tmp);
	}
	return v.toArray(new Node[0]); // 配列に変換して返す
    }
    public String toString() {
	return Node.toString(nodes);
    }
}
